@@ -7,6 +7,10 @@ from mohou.embedding_rule import EmbeddingRule
 from mohou.propagator import Propagator
 
 
+class DefaultNotFoundError(Exception):
+    pass
+
+
 def auto_detect_autoencoder_type(project_name: str) -> Type[AutoEncoderBase]:
     # TODO(HiroIshida) dirty...
     t: Optional[Type[AutoEncoderBase]] = None
@@ -22,7 +26,11 @@ def auto_detect_autoencoder_type(project_name: str) -> Type[AutoEncoderBase]:
         except Exception:
             pass
 
-    assert detect_count == 1
+    if detect_count == 0:
+        raise DefaultNotFoundError("no autoencoder found")
+    if detect_count > 1:
+        raise DefaultNotFoundError("multiple autoencoder found")
+
     assert t is not None  # redundant but for mypy check
     return t
 
@@ -34,8 +42,13 @@ def create_default_embedding_rule(project_name: str) -> EmbeddingRule:
     av_dim = chunk_spec.type_shape_table[AngleVector][0]
     ae_type = auto_detect_autoencoder_type(project_name)
 
-    tcache_autoencoder = TrainCache.load(project_name, ae_type)
-    assert tcache_autoencoder.best_model is not None
+    try:
+        tcache_autoencoder = TrainCache.load(project_name, ae_type)
+    except Exception:
+        raise DefaultNotFoundError("not TrainCache for autoencoder is found ")
+
+    if tcache_autoencoder.best_model is None:
+        raise DefaultNotFoundError("traincache was found but best model is not saved ")
     image_embed_func = tcache_autoencoder.best_model.get_embedder()
     av_idendical_func = IdenticalEmbedder(AngleVector, av_dim)
 
@@ -53,7 +66,11 @@ def create_default_embedding_rule(project_name: str) -> EmbeddingRule:
 
 
 def create_default_propagator(project_name: str) -> Propagator:
-    tcach_lstm = TrainCache.load(project_name, LSTM)
+    try:
+        tcach_lstm = TrainCache.load(project_name, LSTM)
+    except Exception:
+        raise DefaultNotFoundError("not TrainCache for lstm is found ")
+
     embed_rule = create_default_embedding_rule(project_name)
     assert tcach_lstm.best_model is not None
     propagator = Propagator(tcach_lstm.best_model, embed_rule)
