@@ -1,10 +1,10 @@
 from abc import ABC, abstractmethod
-import os
 import copy
 from dataclasses import dataclass, asdict
 import functools
 import operator
 import random
+from pathlib import Path
 import yaml
 from typing import Generic, Optional, List, Tuple, Type, TypeVar, Iterator, Sequence, ClassVar, Dict, Union
 
@@ -13,10 +13,11 @@ import matplotlib.pyplot as plt
 import torch
 import torchvision
 import cv2
+import PIL.Image
 
 from mohou.constant import N_DATA_INTACT
-from mohou.file import get_project_dir, load_object, dump_object
-from mohou.image_randomizer import _f_randomize_rgb_image, _f_randomize_depth_image
+from mohou.file import get_project_path, load_object, dump_object
+from mohou.image_randomizer import _f_randomize_rgb_image, _f_randomize_depth_image, _f_randomize_gray_image
 from mohou.constant import CONTINUE_FLAG_VALUE, TERMINATE_FLAG_VALUE
 from mohou.utils import get_all_concrete_leaftypes
 from mohou.utils import split_sequence, canvas_to_ndarray
@@ -25,6 +26,7 @@ from mohou.utils import assert_with_message, assert_isinstance_with_message
 ElementT = TypeVar('ElementT', bound='ElementBase')
 PrimitiveElementT = TypeVar('PrimitiveElementT', bound='PrimitiveElementBase')
 PrimitiveImageT = TypeVar('PrimitiveImageT', bound='PrimitiveImageBase')
+ColorImageT = TypeVar('ColorImageT', bound='ColorImageBase')
 CompositeImageT = TypeVar('CompositeImageT', bound='CompositeImageBase')
 ImageT = TypeVar('ImageT', bound='ImageBase')
 VectorT = TypeVar('VectorT', bound='VectorBase')
@@ -175,8 +177,7 @@ class PrimitiveImageBase(PrimitiveElementBase, ImageBase):
         return cls._channel
 
 
-class RGBImage(PrimitiveImageBase):
-    _channel: ClassVar[int] = 3
+class ColorImageBase(PrimitiveImageBase, Generic[ColorImageT]):
 
     def __init__(self, data: np.ndarray) -> None:
         super().__init__(data)
@@ -185,6 +186,22 @@ class RGBImage(PrimitiveImageBase):
     def to_tensor(self) -> torch.Tensor:
         return torchvision.transforms.ToTensor()(self._data).float()
 
+    @classmethod
+    def dummy_from_shape(cls: Type[ColorImageT], shape2d: Tuple[int, int]) -> ColorImageT:
+        shape = (shape2d[0], shape2d[1], cls.channel())
+        dummy_array = np.random.randint(0, high=255, size=shape, dtype=np.uint8)
+        return cls(dummy_array)
+
+
+class RGBImage(ColorImageBase['RGBImage']):
+    _channel: ClassVar[int] = 3
+
+    @classmethod
+    def from_file(cls, filename: str) -> 'RGBImage':
+        pil_img = PIL.Image.open(filename).convert('RGB')
+        arr = np.array(pil_img)
+        return cls(arr)
+
     @classmethod
     def from_tensor(cls, tensor: torch.Tensor) -> 'RGBImage':
         tf = torchvision.transforms.ToPILImage()
@@ -196,12 +213,6 @@ class RGBImage(PrimitiveImageBase):
         rand_image_arr = _f_randomize_rgb_image(self._data)
         return RGBImage(rand_image_arr)
 
-    @classmethod
-    def dummy_from_shape(cls, shape2d: Tuple[int, int]) -> 'RGBImage':
-        shape = (shape2d[0], shape2d[1], cls.channel())
-        dummy_array = np.random.randint(0, high=255, size=shape, dtype=np.uint8)
-        return cls(dummy_array)
-
     def to_rgb(self, *args, **kwargs) -> 'RGBImage':
         return self
 
@@ -209,6 +220,44 @@ class RGBImage(PrimitiveImageBase):
         self._data = cv2.resize(self._data, shape2d_new, interpolation=cv2.INTER_AREA)
 
 
+class GrayImage(ColorImageBase['GrayImage']):
+    _channel: ClassVar[int] = 1
+
+    @classmethod
+    def from_tensor(cls, tensor: torch.Tensor) -> 'GrayImage':
+        tensor2d = tensor.squeeze(dim=0)
+        tf = torchvision.transforms.ToPILImage()
+        pil_iamge = tf(tensor2d)
+        return cls(np.expand_dims(np.array(pil_iamge), axis=2))
+
+    def randomize(self) -> 'GrayImage':
+        assert _f_randomize_gray_image is not None
+        rand_image_arr = _f_randomize_gray_image(self._data)
+        return GrayImage(rand_image_arr)
+
+    def to_rgb(self, *args, **kwargs) -> RGBImage:
+        arr = np.array(cv2.cvtColor(self._data[:, :, 0], cv2.COLOR_GRAY2RGB))
+        return RGBImage(arr)
+
+    def resize(self, shape2d_new: Tuple[int, int]) -> None:
+        arr = cv2.resize(self._data, shape2d_new, interpolation=cv2.INTER_AREA)
+        self._data = np.expand_dims(arr, axis=2)
+
+
+def extract_contour_by_laplacian(
+        rgb: RGBImage,
+        laplace_kernel_size: int = 3,
+        blur_kernel_size: Optional[Tuple[int, int]] = None) -> GrayImage:
+
+    if blur_kernel_size is None:
+        blur_kernel_size = (int(rgb.shape[0] * 0.02), int(rgb.shape[1] * 0.02))
+
+    src_gray = cv2.cvtColor(rgb.numpy(), cv2.COLOR_BGR2GRAY)
+    dst = cv2.Laplacian(src_gray, cv2.CV_8U, ksize=laplace_kernel_size)
+    dst2 = cv2.blur(dst, (blur_kernel_size[0], blur_kernel_size[1]))
+    return GrayImage(np.expand_dims(np.uint8(dst2), axis=2))
+
+
 class DepthImage(PrimitiveImageBase):
     _channel: ClassVar[int] = 1
     _max_value: ClassVar[float] = 4.0
@@ -551,16 +600,18 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
         n_type_expected = len(data_list[0].types())
         assert_with_message(n_type_appeared, n_type_expected, 'num of element in chunk')
 
+        data_list_intact = []
+        if with_intact_data:
+            assert N_DATA_INTACT > 0
+            interval = len(data_list) // N_DATA_INTACT
+            indices_intact = [interval * i for i in range(N_DATA_INTACT)]
+            # sorted is necessary because pop changes index
+            for idx in sorted(indices_intact, reverse=True):
+                data_list_intact.append(data_list.pop(idx))
+
         if shuffle:
             random.shuffle(data_list)
 
-        if with_intact_data:
-            data_list_intact = data_list[:N_DATA_INTACT]
-            data_list = data_list[N_DATA_INTACT:]
-        else:
-            data_list_intact = []
-            data_list = data_list
-
         type_shape_table = data_list[0].type_shape_table
 
         # chunk spec
@@ -580,17 +631,18 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
         return chunk
 
     @staticmethod
-    def spec_file_name(project_name: str, postfix: Optional[str] = None) -> str:
+    def spec_file_path(project_name: str, postfix: Optional[str] = None) -> Path:
+        project_path = get_project_path(project_name)
         if postfix is None:
-            yaml_file_name = os.path.join(get_project_dir(project_name), 'chunk_spec.yaml')
+            yaml_file_path = project_path / 'chunk_spec.yaml'
         else:
-            yaml_file_name = os.path.join(get_project_dir(project_name), 'chunk_spec-{}.yaml'.format(postfix))
-        return yaml_file_name
+            yaml_file_path = project_path / 'chunk_spec-{}.yaml'.format(postfix)
+        return yaml_file_path
 
     @classmethod
     def load_spec(cls, project_name: str, postfix: Optional[str] = None) -> ChunkSpec:
-        yaml_file_name = cls.spec_file_name(project_name, postfix)
-        with open(yaml_file_name, 'r') as f:
+        yaml_file_path = cls.spec_file_path(project_name, postfix)
+        with yaml_file_path.open(mode='r') as f:
             d = yaml.safe_load(f)
             spec = ChunkSpec.from_dict(d)
         return spec
@@ -598,8 +650,8 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
     def dump(self, project_name: str, postfix: Optional[str] = None) -> None:
         self._postfix = postfix
         dump_object(self, project_name, postfix)
-        yaml_file_name = self.spec_file_name(project_name, postfix)
-        with open(yaml_file_name, 'w') as f:
+        yaml_file_path = self.spec_file_path(project_name, postfix)
+        with yaml_file_path.open(mode='w') as f:
             yaml.dump(self.spec.to_dict(), f, default_flow_style=False, sort_keys=False)
 
     def get_intact_chunk(self) -> 'MultiEpisodeChunk':
@@ -650,6 +702,8 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
         if project_name is None:
             plt.show()
         else:
-            filename = os.path.join(get_project_dir(project_name), 'seq-{}.png'.format(elem_type.__name__))
-            fig.savefig(filename, format='png', dpi=300)
-            print('saved to {}'.format(filename))
+            project_path = get_project_path(project_name)
+            file_path = project_path / 'seq-{}.png'.format(elem_type.__name__)
+            file_name = str(file_path)
+            fig.savefig(file_name, format='png', dpi=300)
+            print('saved to {}'.format(file_name))
