@@ -1,17 +1,11 @@
-import logging
-from logging import Logger
-import os
 import queue
-import time
-from typing import Any, List, Iterator, TypeVar, Union, Type
+from typing import Any, List, Iterator, TypeVar, Union, Type, Sequence
 
 import numpy as np
 import PIL
 import torch
 from torch.utils.data import Dataset, random_split
 
-from mohou.file import get_project_dir
-
 
 def get_all_concrete_leaftypes(root: Type) -> List[Type]:
     concrete_types: List[Type] = []
@@ -57,25 +51,6 @@ def split_with_ratio(dataset: Dataset, valid_ratio: float = 0.1):
     return ds_train, ds_validate
 
 
-def create_default_logger(project_name: str, prefix: str) -> Logger:
-    timestr = "_" + time.strftime("%Y%m%d%H%M%S")
-    log_dir = os.path.join(get_project_dir(project_name), 'log')
-    if not os.path.exists(log_dir):
-        os.makedirs(log_dir)
-    log_file_name = os.path.join(log_dir, (prefix + timestr + '.log'))
-    FORMAT = '[%(levelname)s] %(asctime)s %(name)s: %(message)s'
-    logging.basicConfig(filename=log_file_name, format=FORMAT)
-    logger = logging.getLogger('mohou')
-    logger.setLevel(level=logging.INFO)
-
-    log_sym_name = os.path.join(log_dir, ('latest_' + prefix + '.log'))
-    logger.info('create log symlink :{0} => {1}'.format(log_file_name, log_sym_name))
-    if os.path.islink(log_sym_name):
-        os.unlink(log_sym_name)
-    os.symlink(log_file_name, log_sym_name)
-    return logger
-
-
 AnyT = TypeVar('AnyT', bound=Any)
 
 
@@ -92,6 +67,13 @@ def assert_isinstance_with_message(given: Any, expected: Type):
     assert isinstance(given, expected), message
 
 
+def assert_two_sequences_same_length(seq_list1: Sequence[np.ndarray], seq_list2: Sequence[np.ndarray]):
+    if __debug__:
+        assert_with_message(len(seq_list1), len(seq_list2), 'seq_list length of lhs')
+        for seq1, seq2 in zip(seq_list1, seq_list2):
+            assert_with_message(len(seq1), len(seq2), 'seq length of lhs')
+
+
 def canvas_to_ndarray(fig, resize_pixel=None) -> np.ndarray:
     fig.canvas.draw()
     data = np.fromstring(fig.canvas.tostring_rgb(), dtype=np.uint8, sep='')
